#include <gint/display.h>
#include <gint/keyboard.h>
#include "main.h"

Vec2 search(tile_t x, tile_t level[16][16]) {
	// Search for x in a given matrix.
	// If x is found, return it coordinates
	Vec2 coordinates = {0, 0};
	for(int m = 0; m < LEVEL_SIZE; ++m) {
			for(int n = 0; n < LEVEL_SIZE; ++n) {
				if(level[m][n] == x) {
					// idk why m and n are inversed but it works kek
					coordinates.x = n * 12;
					coordinates.y = m * 12;
					return coordinates;
				}
			}
		}
	return coordinates;
}

int collide_pixel(Vec2 pos, tile_t obj, tile_t level[LEVEL_SIZE][LEVEL_SIZE]) {
	// Check if there's something in (x, y)
	if(obj == level[pos.x / TILE_SIZE][pos.y / TILE_SIZE]) {
		return 1;
	}
	else {
		return 0;
	}
}

int collide(Vec2 pos, int h, tile_t obj, tile_t level[LEVEL_SIZE][LEVEL_SIZE]) {
	/* tl = top left
	 * br = bottom right
	 * avoid repetition later on ezier to work on */
	const Vec2 pos_tl = (Vec2){pos.x + h, pos.y + h};
	const Vec2 pos_br = (Vec2){pos.x + TILE_SIZE - h - 1, pos.y + TILE_SIZE - h - 1};
	// Check if there's something in
	// the square (x + 1, y + 1, x + 11, y + 11)
	// The size of the hitbox changes with h
	if(collide_pixel(pos_tl, obj, level) ||
	   collide_pixel(pos_br, obj, level) ||
	   collide_pixel((Vec2){pos_tl.x, pos_br.y}, obj, level) ||
	   collide_pixel((Vec2){pos_br.x, pos_tl.y}, obj, level))
	{
		return 1;
	}
	return 0;
}

Player level_reset(Player player) {
	player.pos = player.spawn;
	return player;
}

int main(void) {
	extern bopti_image_t img_player;
	extern bopti_image_t img_wall;
	extern bopti_image_t img_spike;

	extern tile_t level[LEVEL_SIZE][LEVEL_SIZE];

	int running = 1;

	// player
	Player player = {
		.pos = {0, 0},
		.spawn = {0, 0}
	};
	player.spawn = search(2, level);
	player = level_reset(player);

	// main loop
	while(running) {

		dclear(C_BLACK);

		// drawing the level
		for(int m = 0; m < LEVEL_SIZE; ++m) {
			for(int n = 0; n < LEVEL_SIZE; ++n) {
				switch(level[n][m]) {
					case 1:
						// walls
						dimage(m * TILE_SIZE, n * TILE_SIZE, &img_wall);
						break;
					case 3:
						// spikes
						dimage(m * TILE_SIZE, n * TILE_SIZE, &img_spike);
						break;
				}
			}
		}

		// drawing the player
		dimage(player.pos.x, player.pos.y, &img_player);

		dupdate();

		/* if something has x and y, you probably want to use your Vec2 struct */
		Vec2 mov = {
			.x = keydown(KEY_RIGHT) - keydown(KEY_LEFT),
			.y = keydown(KEY_DOWN) - keydown(KEY_UP)
		};
		clearevents();

		// trying to move the player >w<
		if(!collide((Vec2){player.pos.x + mov.x, player.pos.y}, 0, 1, level)) {
			player.pos.x += mov.x;
		}

		if(!collide((Vec2){player.pos.x, player.pos.y + mov.y}, 0, 1, level)) {
			player.pos.y += mov.y;
		}

		// d i e
		if(collide(player.pos, 2, 3, level)) {
			player = level_reset(player);
		}

		if(keydown(KEY_EXIT)) running = 0;
	}

	return 1;
}
